#################################################################################################
# Python script originally written by Marine Brieuc in 2013 and modified by Eleni Petrou in 2018
##################################################################################################
# PURPOSE
# The purpose of this script is to take data files (individual.tags.tsv) produced by pstacks (version 1.29) and 
# use a minimum read depth threshold to designate genotypes. 
##################################################################################################
# Input files:
 #1. A .txt file with the name of each sample stored on each individual line (individual_list.txt). 
 #2. A tab-delimited .txt file with bi-allelic marker information (catalog_file.txt). The first column contains the locus name,
	# the second column contains haplotype1, the third column contains haplotype2, and the following columns contain the positions of the SNPS.
	# example: Locus_10066	AT	TA	74	75
	# PLEASE NOTE - the locus name should match what is in the third column of each individual.tags.tsv file. If you want to match to a different column, edit line 104 : locus_name=line_list[3]
 #3. Individual tags.tsv files from stacks version 1.29  
##################################################################################################
# Output files:
 #1. corrected_genotypes.txt: a tab-delimited dataframe of genotypes, with individual loci stored in each column and individual samples stored in each row. 
 # data are coded as: a = homozygote for haplotype1, b = homozygote for haplotype2, ab = heterozygote, "-" = missing data. 

 
 
###################################################################################################
# PARAMETERS TO EDIT
##########################################################################################
# Specify input files

### Open the file with the list of the individuals
list_of_indiv=open('individual_list.txt','r')

### open the catalog file with name of locus in first column, haplotype1 and haplotype2 in second column, SNP positions in the following columns
Catalog_file=open('catalog_file.txt','r')
########################################################
# Specify output files
genotype_file=open('corrected_genotypes.txt' ,'w')

##########################################################
# Specify minimum allele sequencing depth
min_allele_depth = 1

# Specify minimum locus sequencing depth
min_locus_depth = 9


#########################################################################################
#SCRIPT
##########################################################################################
###Initiate dictionary: locus :: haplotype1,haplotype2
loci={}
for line in Catalog_file:
	line_list=line.split('\t')
	locus=line_list[0]
	line_list[-1]=line_list[-1].split('\n')[0]
	loci[locus] = line_list[1:]

### create a list of loci that will be used later and sort the list 	
list_loci=loci.keys()
list_loci.sort()
#print list_loci

### Add a tab at the beginning of the output file and write all the loci (in order) on the first row, separated by a tab
genotype_file.write('\t')
for locus in list_loci:
	genotype_file.write("%s\t" %locus)
genotype_file.write('\n')

for indiv in list_of_indiv:
	indiv_name=indiv.split('\n')[0]	
	#print indiv_name
	### Open the individual.tags.tsv file
	Indiv_file=open(indiv_name + '.tags.tsv','r')	
	#print Indiv_file
	line_number=0	
	count_haplotype1=0
	count_haplotype2=0
	count_other=0
	keep_locus=0
	locus_name=""
	
	### create a few lists
	list_to_export=list()
	loci_found=list()
	
	### for each line in the individual.tags.tsv file
	for line in Indiv_file:
		line_number+=1		
		if "consensus" in line:	
			### if line has consensus in it, export information about the previous locus (because you know that you're done with the previous locus now that you've encountered a new locus_	
			if keep_locus ==1:
				### call genotypes, based on depth ### you can change the depth
				if (count_haplotype1 > min_allele_depth and count_haplotype2 > min_allele_depth and count_haplotype1+count_haplotype2> min_locus_depth):
					genotype="ab"
				elif (count_haplotype2 > min_allele_depth and count_haplotype1+count_haplotype2> min_locus_depth):
					genotype="b"
				elif (count_haplotype1 > min_allele_depth and count_haplotype1+count_haplotype2> min_locus_depth):
					genotype="a"
				else:
					genotype="-"				
				list_to_export.append([locus_name,indiv_name,genotype])
			### Reinitialize the parameters
			count_haplotype1=0
			count_haplotype2=0
			count_other=0			
			line_list=line.split('\t')
			locus_name=line_list[3]
			### If locus in the catalog, extract haplotype and SNP positions
			if locus_name in list_loci and "94\t-" not in line: # 94/t is an "error flag" that stacks uses
				loci_found.append(locus_name)
				keep_locus = 1
				locus_line = line_number
				haplotype1=loci[locus_name][0]
				haplotype2=loci[locus_name][1]
				position= list()
				RANGE = range(2,len(haplotype1)+2,1)
				#print locus_name
				#print list_loci
				for index in RANGE:
					position.append(loci[locus_name][index])
					#print position
					#print keep_locus

			else:
				keep_locus=0
		
		### If line doesn't have "consensus in it, look for haplotypes in the reads if it is a locus that is in the catalog
		elif (keep_locus==1 and line_number>(locus_line+1)):
			haplotype=""
			line_list=line.split('\t')			
			sequence=line_list[9]
			#print position[0]
			for where in position:
				haplotype=haplotype+sequence[int(where)]
			#print haplotype
			if haplotype==haplotype1:
				count_haplotype1 += 1
			elif haplotype==haplotype2:
				count_haplotype2 += 1
			else:
				count_other += 1
	
	### When at the end of the file, call the genotype if the last locus of the file was in the catalog
	if keep_locus ==1:
		if (count_haplotype1 > min_allele_depth and count_haplotype2 > min_allele_depth and count_haplotype1+count_haplotype2> min_locus_depth):
			genotype="ab"
		elif (count_haplotype2 > min_allele_depth and count_haplotype1+count_haplotype2> min_locus_depth):
			genotype="b"
		elif (count_haplotype1 > min_allele_depth and count_haplotype1+count_haplotype2> min_locus_depth):
			genotype="a"
		else:
			genotype="-"					
		list_to_export.append([locus_name,indiv_name,genotype])
	
	### Identify all loci that were missing in the individual and write them as missing value
#	print list_to_export	
	loci_missing = list(set(list_loci)-set(loci_found))
	#print loci_missing
	for locus in loci_missing:
		list_to_export.append([locus,indiv_name,"-"])	
	### At this point, sort by locus, all the loci in the dictionary should also be in this list
	list_to_export.sort()
	#print len(list_to_export)
	#print len(list_loci)
#	print list_to_export
	#print loci_missing
	
	for i in range(1, len(list_loci)):
		if list_loci[i] != list_to_export[i][0]:
			print list_loci[i], list_to_export[i], i
			
	
	### If the list of loci in the dictionary = list of loci to export (it should, but it's just to double check), write the genotypes in the output file (each line = indiv_name followed by the genotypes
	loci_in_order=list()
	genotype_in_order=""

	
	for locus in list_to_export:
		loci_in_order.append(locus[0])
		genotype_in_order=genotype_in_order + locus[2] + '\t'
	#print loci_in_order[0:-1] == list_loci
	if loci_in_order==list_loci:
		print "same order"		
		genotype_file.write(indiv_name + '\t' + genotype_in_order + '\n')
	else: 
		print "different - WTF!!!!"	
			
	Indiv_file.close()
genotype_file.close()
list_of_indiv.close()
Catalog_file.close()
	
